/*!\file METIS_PartMeshNodalPatch
 * \brief common interface to Metis 4.0 and 5.0
 */

#include <config.h>
#include "../metisincludes.h"
#include "../../../shared/shared.h"

void METIS_PartMeshNodalPatch(int* pnumberofelements,int* pnumberofnodes, int* index, int* petype, int* pnumflag, int* pnum_procs, int* pedgecut, int* epart, int* npart){

	#if _METIS_VERSION_ == 4
	/*Our interface originates in the Metis 4.0 version, hence identical calls*/
	METIS_PartMeshNodal(pnumberofelements,pnumberofnodes, index, petype, pnumflag, pnum_procs, pedgecut, epart, npart); 
	#elif _METIS_VERSION_ == 5

	/*This interface is heavily changed. More options, different ways of meshing, etc ...: */ 

	/*Create options*/
	idx_t options[METIS_NOPTIONS];
	METIS_SetDefaultOptions(options);

	options[METIS_OPTION_PTYPE]   = METIS_PTYPE_KWAY;     /* partitioning method  */
	options[METIS_OPTION_OBJTYPE] = METIS_OBJTYPE_VOL;    /* type of objective */
	options[METIS_OPTION_CTYPE]   = METIS_CTYPE_SHEM;     /* matching scheme to be used during coarsening.*/
	options[METIS_OPTION_IPTYPE]  = METIS_IPTYPE_METISRB; /* algorithm used during initial partitioning*/
	options[METIS_OPTION_RTYPE]   = METIS_RTYPE_GREEDY;   /* algorithm used for refinement*/
	options[METIS_OPTION_DBGLVL]  = 0;                    /* amount of progress/debugging information will be printed */
	options[METIS_OPTION_UFACTOR] = 30;                   /* maximum allowed load imbalance among the partitions*/
	options[METIS_OPTION_MINCONN] = 0;                    /* explicitly minimize the maximum connectivity ?*/
	options[METIS_OPTION_CONTIG]  = 0;                    /* force contiguous partitions?*/
	options[METIS_OPTION_SEED]    = -1;                   /* seed for the random number generator*/
	options[METIS_OPTION_NITER]   = 10;                   /* number of iterations for the refinement algorithms*/
	options[METIS_OPTION_NCUTS]   = 1;                    /* number of different partitionings that it will compute*/

	/*create eptr: */
	idx_t  k=0;
	idx_t* eptr=xNew<idx_t>((*pnumberofelements+1));
	eptr[0]=0;
	for(int i=0;i<*pnumberofelements;i++){
		k+=3;
		eptr[i+1]=k;
	}

	/*create tpwgts: */
	real_t* tpwgts=xNew<real_t>(*pnum_procs);
	for(int i=0;i<*pnum_procs;i++){
		tpwgts[i]=1.0/(*pnum_procs);
	}

	/*Call METIS*/
	idx_t objval;
	METIS_PartMeshNodal(pnumberofelements,pnumberofnodes, eptr, index,
			NULL, NULL, pnum_procs, tpwgts, options, &objval,epart, npart);

	/*clean-up*/
	xDelete<idx_t>(eptr);
	xDelete<real_t>(tpwgts);

	#else
	_error_("METIS version not supported yet");
	#endif
}
